class Tree(list):

    def __init__(self, node, children):
        list.__init__(self, children)
        self.node = node

    def __getitem__(self, index):
        if isinstance(index, int):
            return list.__getitem__(self, index)
        else:
            if len(index) == 0:
                return self
            elif len(index) == 1:
                return self[int(index[0])]
            else:
                return self[int(index[0])][index[1:]]
    

    def __repr__(self):
        import string
        childstr = string.join([repr(c) for c in self])
        return '(%s: %s)' % (self.node, childstr)
    def __str__(self):
        return self.pp()

    def leaves(self):
        leaves = []
        for child in self:
            if isinstance(child, Tree):
                leaves.extend(child.leaves())
            else:
                leaves.append(child)
        return leaves

    def height(self):
        max_child_height = 0
        for child in self:
            if isinstance(child, Tree):
                max_child_height = max(max_child_height, child.height())
            else:
                max_child_height = max(max_child_height, 1)
        return 1 + max_child_height

    def subtrees(self, filter=None):
        if not filter or filter(self):
            yield self
        for child in self:
            if isinstance(child, Tree):
                for subtree in child.subtrees(filter):
                    yield subtree

from nltk_lite.corpora import brown
count = 0
total = 0
for sent in brown.raw('a'):
    for token in sent:
        count += 1
        total += len(token)
print float(total) / count

tokens = [token for sent in brown.raw('a') for token in sent]
total = sum(map(len, tokens))
print float(total)/len(tokens)

word_list = []
for sent in brown.raw('a'):
    for token in sent:
        if token not in word_list:
            word_list.append(token)
word_list.sort()

tokens = [word for sent in brown.raw('a') for word in sent]
word_list = list(set(tokens))
word_list.sort()

def reverse(word):
    return word[::-1]

from nltk_lite.corpora import words
def reverse_cmp(x,y):
    return cmp(reverse(x), reverse(y))
word_list = [word for word in words.raw('en') if len(word) == 3]
word_list.sort(reverse_cmp)
print word_list[-12:]

word_list = []
for word in words.raw('en'):
    if len(word) == 3:
        word_list.append(reverse(word))
word_list.sort()
for i in range(len(word_list)):
    word_list[i] = reverse(word_list[i])
print word_list[-12:]

import pdb
import mymodule
pdb.run('mymodule.test()')

from random import shuffle
a = range(1000)                     # [0,1,2,...999]
shuffle(a)                          # randomize

from nltk_lite.misc import sort
sort.bubble(a)

shuffle(a); sort.merge(a)

shuffle(a); sort.quick(a)

words = 'I turned off the spectroroute'.split()
words.sort(cmp)
words

words.sort(lambda x, y: cmp(len(y), len(x)))
words

[pair[1] for pair in sorted((len(w), w) for w in words)[::-1]]

from timeit import Timer
Timer("sorted(words, lambda x, y: cmp(len(y), len(x)))",
      "words='I turned off the spectroroute'.split()").timeit()

Timer("[pair[1] for pair in sorted((len(w), w) for w in words)]",
     "words='I turned off the spectroroute'.split()").timeit()

from nltk_lite.corpora import words
for word1 in words.raw():
    for word2 in words.raw():
        if word1 == word2[::-1]:
            print word1

from nltk_lite.corpora import words
wordlist = set(words.raw())
rev_wordlist = set(w[::-1] for w in wordlist)
sorted(wordlist.intersection(rev_wordlist))

words = ["sequoia", "abacadabra", "yiieeaouuu!"]
vowels = "aeiou"
[w for w in words if set(w).issuperset(vowels)]

[w for w in words if sorted(c for c in w if c in vowels) == list(vowels)]

text = "doyouseethekittyseethedoggydoyoulikethekittylikethedoggy"
seg1 = "0000000000000001000000000010000000000000000100000000000"
seg2 = "0100100100100001001001000010100100010010000100010010000"

segment(text, seg1)

segment(text, seg2)

text = "doyouseethekittyseethedoggydoyoulikethekittylikethedoggy"
seg3 = "0000100100000011001000000110000100010000001100010000001"
segment(text, seg3)

evaluate(text, seg3)

print evaluate(text, seg1), segment(text, seg1)

hill_climb(text, segs1, 20)

anneal(text, segs, 5000, 1.2)

from nltk_lite.probability import FreqDist
from nltk_lite import tokenize
def freq_words(file, min, num):
    freqdist = FreqDist()
    text = open(file).read()
    for word in tokenize.wordpunct(text):
        if len(word) >= min:
            freqdist.inc(word)
    return freqdist.sorted_samples()[:num]
freq_words('programming.txt', 4, 10)

def freq_words(file, min=1, num=10):
    freqdist = FreqDist()
    text = open(file).read()
    for word in tokenize.wordpunct(text):
        if len(word) >= min:
            freqdist.inc(word)
    return freqdist.sorted_samples()[:num]

freq_words('programming.txt', 4, 10)

freq_words('programming.txt', min=4, num=10)

freq_words('programming.txt', num=10, min=4)

freq_words('programming.txt', min=4)

freq_words('programming.txt', 4)

def freq_words(file, min=1, num=10, trace=False):
    freqdist = FreqDist()
    if trace: print "Opening", file
    text = open(file).read()
    if trace: print "Read in %d characters" % len(file)
    for word in tokenize.wordpunct(text):
        if len(word) >= min:
            freqdist.inc(word)
            if trace and freqdist.N() % 100 == 0: print "."
    if trace: print
    return freqdist.sorted_samples()[:num]

def find_nouns(tagged_text):
    nouns = []
    for word, tag in tagged_text:
        if tag[:2] == 'NN':
            nouns.append(word)
    return nouns

tagged_text = [('the', 'DT'), ('cat', 'NN'), ('sat', 'VBD'),
               ('on', 'IN'), ('the', 'DT'), ('mat', 'NN')]
find_nouns(tagged_text)

def find_nouns(tagged_text):
    for word, tag in tagged_text:
        if tag[:2] == 'NN':
            yield word

find_nouns(tagged_text)

list(find_nouns(tagged_text))

for noun in find_nouns(tagged_text):
    print noun,

set1 = set(['a', 'b', 1, 2, 3])
set1

'a' in set1

'c' in set1

set2 = set([3, 2, 1, 'b', 'a'])
set1 == set2

len(set1)

set('123')

a = set()
b = set()
a == b

odds = set('13579')
evens = set('02468')
numbers = odds | evens
numbers

ints

ints & nats

odds & evens

nats - ints

odds == nats - evens

odds == odds - set()

nats = set(range(10))
evens1 = set([n for n in nats if n % 2 == 0])
evens1

evens2 = set([int(n) for n in evens])
evens1 == evens2

evens1 <= nats

set() <= nats

evens1 <= evens1

evens1 < nats

evens1 < evens1

a = frozenset('a')
aplus = set([a])
aplus

tup1 = ('a', 'b', 'c')
tup2 = ('a', 'c', 'b')
tup1 == tup2

A = set([1, 2, 3])
B = set('ab')
AxB = set([(a, b) for a in A for b in B])
AxB

AxA = set([(a1, a2) for a1 in A for a2 in A])
AxA

evens = set([2, 4, 6, 8, 10])
mod = set([(m,n) for m in evens for n in evens if n % m == 0 and m < n])
mod

odds = set([1, 2, 3, 4, 5])
doubles = set([(m,n) for m in odds for n in evens if n == m * 2])
doubles

d = {}
d.update(doubles)
d

doubles2 = set([(m,n) for m in evens for n in evens if n == m * 2])
doubles2

doubles = set([(m,m*2) for m in evens])

