from nltk_lite import tag
from nltk_lite.corpora import brown
from itertools import islice
train_sents = list(islice(brown.tagged(), 500))  # sents 0..499
unigram_tagger = tag.Unigram()
unigram_tagger.train(train_sents)
acc = tag.accuracy(unigram_tagger, train_sents)
print 'Accuracy = %4.1f%%' % (100 * acc)

train_sents  = list(brown.tagged('a'))[:500]
unseen_sents = list(brown.tagged('a'))[500:600] # sents 500-599

unigram_tagger = tag.Unigram(backoff=tag.Default('nn'))
unigram_tagger.train(train_sents)
acc = tag.accuracy(unigram_tagger, unseen_sents)
print 'Accuracy = %4.1f%%' % (100 * acc)

baseline_tagger = tag.Default('nn')
acc = tag.accuracy(baseline_tagger, brown.tagged('a'))
print 'Accuracy = %4.1f%%' % (100 * acc)

from nltk_lite.probability import *
wordcounts = FreqDist()
wordtags = ConditionalFreqDist()
for sent in brown.tagged('a'):
    for (w,t) in sent:
        wordcounts.inc(w)    # count the word
        wordtags[w].inc(t)   # count the word's tag
frequent_words = wordcounts.sorted_samples()[:1000]

table = dict((word, wordtags[word].max()) for word in frequent_words)
baseline_tagger = tag.Lookup(table, tag.Default('nn'))
acc = tag.accuracy(baseline_tagger, brown.tagged('a'))
print 'Accuracy = %4.1f%%' % (100 * acc)

errors = {}
for i in range(len(unseen_sents)):
    raw_sent = tag.untag(unseen_sents[i])
    test_sent = list(unigram_tagger.tag(raw_sent))
    unseen_sent = unseen_sents[i]
    for j in range(len(test_sent)):
        if test_sent[j][1] != unseen_sent[j][1]:
            test_context = test_sent[j-1:j+1]
            gold_context = unseen_sent[j-1:j+1]
            if None not in test_context:
                pair = (tuple(test_context), tuple(gold_context))
                if pair not in errors:
                    errors[pair] = 0
                errors[pair] += 1

counted_errors = [(errors[k], k) for k in errors.keys()]
counted_errors.sort()
counted_errors.reverse()
for err in counted_errors[:5]:
    print err



from nltk_lite import chunk
correct = chunk.tagstr2tree(
   "[ the/DT little/JJ cat/NN ] sat/VBD on/IN [ the/DT mat/NN ]")
correct.flatten()

from nltk_lite.chunk import *
chunkscore = ChunkScore()
rule = ChunkRule('<PRP|DT|POS|JJ|CD|N.*>+',
             "Chunk items that often occur in NPs")
chunkparser = RegexpChunk([rule], chunk_node='NP')
guess = chunkparser.parse(correct.flatten())
chunkscore.score(correct, guess)
print chunkscore

from itertools import islice
from nltk_lite.corpora import treebank
rule = ChunkRule('<DT|JJ|NN>+', "Chunk sequences of DT, JJ, and NN")
chunkparser = RegexpChunk([rule], chunk_node='NP')
chunkscore = ChunkScore()
for chunk_struct in islice(treebank.chunked(), 10):
    test_sent = chunkparser.parse(chunk_struct.flatten())
    chunkscore.score(chunk_struct, test_sent)
print chunkscore

from random import shuffle
missed = chunkscore.missed()
shuffle(missed)
print missed[:10]

incorrect = chunkscore.incorrect()
shuffle(incorrect)

chunkscore = ChunkScore(max_fp_examples=1000,
                        max_fn_examples=1000)

from nltk_lite.probability import ConditionalFreqDist
from nltk_lite.parse import Tree
import re
cfdist = ConditionalFreqDist()
chunk_data = list(treebank.chunked())
split = len(chunk_data)*9/10
train, test = chunk_data[:split], chunk_data[split:]
for chunk_struct in train:
    for constituent in chunk_struct:
        if isinstance(constituent, Tree):
            for (word, tag) in constituent.leaves():
                cfdist[tag].inc(True)
        else:
            (word, tag) = constituent
            cfdist[tag].inc(False)

chunk_tags = [tag for tag in cfdist.conditions() if cfdist[tag].max() == True]
chunk_tags = [re.sub(r'(\W)', r'\\\1', tag) for tag in chunk_tags]
tag_pattern = '<' + '|'.join(chunk_tags) + '>+'
print 'Chunking:', tag_pattern

rule = ChunkRule(tag_pattern, 'Chunk any sequence involving commonly chunked tags')
chunkparser = RegexpChunk([rule], chunk_node='NP')
chunkscore = ChunkScore()
for chunk_struct in test:
    test_sent = chunkparser.parse(chunk_struct.flatten())
    chunkscore.score(chunk_struct, test_sent)
print chunkscore

from nltk_lite.contrib.ieer_rels import *
from itertools import islice
IN = re.compile(r'.*\bin\b(?!\b.+ing\b)')
ieer_trees = (d['text'] for d in ieer.dictionary())
for r in islice(ieer_trees, relextract('ORG', 'LOC', pattern = IN), 29, 39): 
    print show_tuple(r)

vnv = """
(
is/V|
was/V|
werd/V|
wordt/V
)
.*
van/Prep
"""
VAN = re.compile(vnv, re.VERBOSE)
for r in relextract('PER', 'ORG', corpus='conll2002-ned', pattern = VAN): 
    print show_tuple(r)

