tok = ('fly', 'nn')
tok

tok[0]

tok[1]

from nltk_lite.tag import tag2tuple
tag2tuple('fly/nn')

from nltk_lite import tokenize
sent = '''
The/at grand/jj jury/nn commented/vbd on/in a/at number/nn of/in
other/ap topics/nns ,/, among/in them/ppo the/at Atlanta/np and/cc
Fulton/np-tl County/nn-tl purchasing/vbg departments/nns which/wdt it/pps
said/vbd ``/`` are/ber well/ql operated/vbn and/cc follow/vb generally/rb
accepted/vbn practices/nns which/wdt inure/vb to/in the/at best/jjt
interest/nn of/in both/abx governments/nns ''/'' ./.
'''
tagged_words = []                                   # [_tag2tuple1]
for t in tokenize.whitespace(sent):
    tagged_words.append(tag2tuple(t))
tagged_words

[tag2tuple(t) for t in tokenize.whitespace(sent)]   # [_tag2tuple2]

from nltk_lite.corpora import brown, extract
extract(6, brown.tagged('a'))

from nltk_lite.probability import FreqDist
fd = FreqDist()
for sent in brown.tagged():
    for word, tag in sent:
        if tag[:2] == 'vb':
            fd.inc(word+"/"+tag)
fd.sorted_samples()[:20]

tagdict = findtags('nn', brown.tagged('a'))
for tag in sorted(tagdict):
    print tag, tagdict[tag]

fd = FreqDist()
for sent in brown.tagged('a'):
    for word, tag in sent:
        fd.inc(tag)
fd.max()

from nltk_lite import tag
tokens = tokenize.whitespace('John saw 3 polar bears .')
default_tagger = tag.Default('nn')
list(default_tagger.tag(tokens))

tag.accuracy(default_tagger, brown.tagged('a'))

patterns = [
    (r'.*ing$', 'vbg'),               # gerunds
    (r'.*ed$', 'vbd'),                # simple past
    (r'.*es$', 'vbz'),                # 3rd singular present
    (r'.*ould$', 'md'),               # modals
    (r'.*\'s$', 'nn$'),               # possessive nouns
    (r'.*s$', 'nns'),                 # plural nouns
    (r'^-?[0-9]+(.[0-9]+)?$', 'cd'),  # cardinal numbers
    (r'.*', 'nn')                     # nouns (default)
]

regexp_tagger = tag.Regexp(patterns)
list(regexp_tagger.tag(brown.raw('a')))[3]

tag.accuracy(regexp_tagger, brown.tagged('a'))

frequent_words = wordcounts(brown.raw('a'), 100)
frequent_words

[(w,t) for sent in brown.tagged('a')
       for (w,t) in sent
       if w in frequent_words]

table = wordtags(brown.tagged('a'), frequent_words)
table['The']

baseline_tagger = tag.Lookup(table)
tag.accuracy(baseline_tagger, brown.tagged('a'))

list(baseline_tagger.tag(brown.raw('a')))[3]

baseline_tagger = tag.Lookup(table, backoff=tag.Default('nn'))
tag.accuracy(baseline_tagger, brown.tagged('a'))

from pylab import *
sizes = 2**arange(15)
perfs = [performance(size) for size in sizes]
plot(sizes, perfs, '-bo')
title('Lookup Tagger Performance with Varying Model Size')
xlabel('Model Size')
ylabel('Performance')
show()

from nltk_lite.corpora import brown
from itertools import islice
train_sents = list(islice(brown.tagged(), 500))  # sents 0..499
unigram_tagger = tag.Unigram()
unigram_tagger.train(train_sents)

text = "John saw the book on the table"
tokens = list(tokenize.whitespace(text))
list(unigram_tagger.tag(tokens))

affix_tagger = tag.Affix(-2, 3)
affix_tagger.train(train_sents)
list(affix_tagger.tag(tokens))

bigram_tagger = tag.Bigram()
bigram_tagger.train(brown.tagged(['a','b']))

text = "John saw the book on the table"
tokens = list(tokenize.whitespace(text))
list(bigram_tagger.tag(tokens))

t0 = tag.Default('nn')
t1 = tag.Unigram(backoff=t0)
t2 = tag.Bigram(backoff=t1)
t1.train(brown.tagged('a'))    # section a: press-reportage
t2.train(brown.tagged('a'))

accuracy0 = tag.accuracy(t0, brown.tagged('b')) # section b: press-editorial
accuracy1 = tag.accuracy(t1, brown.tagged('b'))
accuracy2 = tag.accuracy(t2, brown.tagged('b'))

print 'Default Accuracy = %4.1f%%' % (100 * accuracy0)

print 'Unigram Accuracy  = %4.1f%%' % (100 * accuracy1)

print 'Bigram Accuracy = %4.1f%%' % (100 * accuracy2)

from nltk_lite.corpora import brown
from nltk_lite.probability import ConditionalFreqDist
cfdist = ConditionalFreqDist()
for sent in brown.tagged('a'):
    p = [(None, None)] # empty token/tag pair
    trigrams = zip(p+p+sent, p+sent+p, sent+p+p)
    for (pair1,pair2,pair3) in trigrams:
        context = (pair1[1], pair2[1], pair3[0]) # last 2 tags, this word
        cfdist[context].inc(pair3[1])            # current tag
total = ambiguous = 0
for cond in cfdist.conditions():
    if cfdist[cond].B() > 1:
        ambiguous += cfdist[cond].N()
    total += cfdist[cond].N()
print float(ambiguous) / total

from nltk_lite.evaluate import ConfusionMatrix
from nltk_lite.tag import untag
def tag_list(tagged_sents):
    return [tag for (word, tag) in sent for sent in tagged_sents]
def apply_tagger(tagger, corpus):
    return [tagger.tag(untag(sent)) for sent in corpus]
gold = tag_list(brown.tagged('b'))
test = tag_list(apply_tagger(t2, brown.tagged('b')))
print ConfusionMatrix(gold, test)

mapping = {'pps': 'nn', 'ppss': 'nn'}
data = [(word, mapping.get(tag, tag))

from cPickle import dump
output = open('t2.pkl', 'wb')
dump(t2, output, -1)
output.close()

from cPickle import load
input = open('t2.pkl', 'rb')
tagger = load(input)
input.close()

text = """The board's action shows what free enterprise
    is up against in our complex maze of regulatory laws ."""
tokens = list(tokenize.whitespace(text))
list(tagger.tag(tokens))

